/*
 * $Id: wbpattern.c,v 1.2 2006/03/22 19:21:45 piru Exp $
 */
#define AROS_ALMOST_COMPATIBLE
#include "global.h"
#include <proto/iffparse.h>
#include <clib/alib_protos.h>
#include <aros/asmcall.h>
#include <exec/libraries.h>

#define DEBUG 0
#include <aros/debug.h>

struct PatternInfo
{
	struct BitMap	*BitMap;
	UWORD		Width;
	UWORD		Height;
	UBYTE		Flags;
	UBYTE		Action;
	Object		*DTObject;
	char		*Name;
};

#define	ACTIONF_UPDATE	0x1

struct PatternInfo	PatternTable[WBP_SCREEN+1];

struct BitMap	*SetImage(ULONG		Type);
static void	FreePatternInfo(ULONG	Type);


AROS_UFH1(LONG, WBPatternPrefs_Handler,
		  AROS_UFHA(struct IFFHandle *, iff, A2))
{
	AROS_LIBFUNC_INIT

	struct WBPatternPrefs	iprefs;
	struct BitMap		*MyBitMap;
	//void			*MyDTObject;
	char			*MyName;
	ULONG			Type;

	D(bug("IPrefs: WBPatternPrefs\n"));

	if (WorkbenchBase->lib_Version >= 50)
		return (0);

	if (ReadChunkBytes(iff, &iprefs, sizeof(iprefs)) == sizeof(iprefs))
	{
		D(bug("IPrefs: Which 0x%lx Flags 0x%lx Revision %ld Depth %ld DataLength 0x%lx\n",
		      iprefs.wbp_Which,
		      iprefs.wbp_Flags,
		      iprefs.wbp_Revision,
		      iprefs.wbp_Depth,
		      iprefs.wbp_DataLength));

		Type = iprefs.wbp_Which;
		if (Type <= WBP_SCREEN)
		{
			if (iprefs.wbp_Flags & WBPF_PATTERN)
			{
				D(bug("IPrefs: WBPF_PATTERN\n"));
				if ((MyBitMap = AllocBitMap(PAT_WIDTH,
				                            PAT_HEIGHT,
				                            iprefs.wbp_Depth,
				                            BMF_CLEAR,
				                            NULL)))
				{
					int k;

					for (k = 0; k < iprefs.wbp_Depth; ++k)
					{
						ReadChunkBytes(iff, MyBitMap->Planes[k], PAT_HEIGHT * ((PAT_WIDTH >> 4) << 1));
					}

					FreePatternInfo(Type);

					PatternTable[Type].Width		=	PAT_WIDTH;
					PatternTable[Type].Height		=	PAT_HEIGHT;
					PatternTable[Type].BitMap		=	MyBitMap;
					PatternTable[Type].Flags		=	iprefs.wbp_Flags;
					SetImage(Type);

					D(bug("IPrefs: OldPattern\n"));
				}
				else
				{
					D(bug("IPrefs: failed to create a Bitmap\n"));

					FreePatternInfo(Type);

					PatternTable[Type].Width		=	0; /* Makes SetImage() pass a null-pointer */
					SetImage(Type);
				}
			}
			else
			{
				D(bug("IPrefs: !WBPF_PATTERN\n"));
				if ((MyName=AllocVec(iprefs.wbp_DataLength + 1,
				                     MEMF_PUBLIC | MEMF_CLEAR)))
				{
					if (ReadChunkBytes(iff, MyName, iprefs.wbp_DataLength) == iprefs.wbp_DataLength)
					{
						D(bug("IPrefs: <%s>\n",
						      MyName));
						if ((PatternTable[Type].Name == NULL) ||
						        ((PatternTable[Type].Name && strcmp(PatternTable[Type].Name,MyName) != 0 )))
						{
							D(bug("IPrefs: Free Image\n"));
							FreePatternInfo(Type);
							PatternTable[Type].Name		=	MyName;
							PatternTable[Type].Flags		=	iprefs.wbp_Flags;
							PatternTable[Type].Action		|=	ACTIONF_UPDATE;
							/*
							 * Notify mainloop to load patterns after all prefs were loaded
							 */
							Signal(&HandlerProcess->pr_Task,
							       SIGBREAKF_CTRL_D);

							/*
							 * The pointer is in use now, don't free it
							 */
							MyName = NULL;
						}
						else
						{
							D(bug("IPrefs: Same Path\n"));
						}
					}
					else
					{
						D(bug("IPrefs: name read error\n"));
					}

					if (MyName)
						FreeVec(MyName);
				}
				else
				{
					D(bug("IPrefs: can't alloc name\n"));
				}
			}
		}
		else
		{
			D(bug("IPrefs: which %ld field is unknown\n",iprefs.wbp_Which));
		}
	}
	else
	{
		D(bug("IPrefs: can't read prefs\n"));
	}

	return 0;

	AROS_LIBFUNC_EXIT
}

struct Library	*ObtainDTBase(void)
{
	if (DataTypesBase==NULL)
	{
		if ((DataTypesBase=OpenLibrary("datatypes.library",0)))
		{}

	}
	return(DataTypesBase);
}

void	ReleaseDTBase(void)
{
	if (DataTypesBase)
	{
		CloseLibrary(DataTypesBase);
		DataTypesBase	=	NULL;
	}
}

struct BitMap	*SetImage(ULONG		Type)
{
	struct BitMap *OldBitMap;

	D(bug("IPrefs_SetImage: Type %ld\n",
	      Type));

	if (Type < WBP_SCREEN)
	{
		D(bug("IPrefs_SetImage: Normal WB Image\n"));
		if (PatternTable[Type].Width == 0 || PatternTable[Type].Height == 0)
			OldBitMap = WBConfig(1 + Type, NULL);
		else
			OldBitMap = WBConfig(1 + Type, &PatternTable[Type].BitMap);
	}
	else
	{
		D(bug("IPrefs_SetImage: Screen Image\n"));
		OldBitMap = NULL;
	}
	D(bug("IPrefs_SetImage: OldBitMap 0x%lx\n",
	      OldBitMap));
	return(OldBitMap);
}

void	*LoadImage(struct Screen	*MyScreen,
				ULONG		Type)
{
	void			*MyObject = NULL;
	struct gpLayout		gpLay;
	struct BitMapHeader	*bmh = 0;
	struct BitMap		*MyBitMap;

	D(bug("IPrefs_LoadImage: MyScreen 0x%lx Type %ld\n",
	      MyScreen,
	      Type));

	if (ObtainDTBase())
	{
		if ((MyObject = NewDTObject(PatternTable[Type].Name,
		                            DTA_GroupID, GID_PICTURE,
		                            PDTA_Remap, FALSE,
		                            PDTA_Screen, (ULONG)MyScreen,
		                            PDTA_DestBitMap, (ULONG)MyScreen->RastPort.BitMap,
		                            PDTA_DestMode, PMODE_V43,
		                            PDTA_UseFriendBitMap, TRUE,
		                            PDTA_FreeSourceBitMap, TRUE,
		                            TAG_DONE)))
		{
			D(bug("IPrefs_LoadImage: Object 0x%lx\n",
			      MyObject));
			gpLay.MethodID	=	DTM_PROCLAYOUT;
			gpLay.gpl_GInfo	=	NULL;
			gpLay.gpl_Initial	=	1L;
			if (DoMethodA(MyObject,(Msg)&gpLay))
			{
				if (GetDTAttrs(MyObject,
				               PDTA_BitMapHeader,(ULONG)&bmh,
				               PDTA_BitMap,(ULONG)&MyBitMap,
				               NULL))
				{
					D(bug("IPrefs_LoadImage: Pattern BitMap 0x%lx\n",
					      MyBitMap));
					D(bug("IPrefs_LoadImage: Width %ld Height %ld\n",
					      bmh->bmh_Width,
					      bmh->bmh_Height));

					//          D(bug("IPrefs_LoadImage: BitMap Type 0x%lx\n",GetCyberMapAttr(bm,CYBRMATTR_PIXFMT)));
					PatternTable[Type].Width	=	bmh->bmh_Width;
					PatternTable[Type].Height	=	bmh->bmh_Height;
					PatternTable[Type].DTObject	=	MyObject;
					PatternTable[Type].BitMap	=	MyBitMap;
#if 0
					if (PatternTable[Type].BitMap = AllocBitMap(PatternTable[Type].Width,
					                                PatternTable[Type].Height,
					                                bmh->bmh_Depth,
					                                BMF_MINPLANES,
					                                MyScreen->RastPort.BitMap))
					{
						D(bug("IPrefs_LoadImage: Friend Pattern BitMap 0x%lx\n",PatternTable[Type].BitMap));
						/* a friend BitMap created...*/
						if (MyScreen->Height != bmh->bmh_Height)
						{
							struct BitScaleArgs bsa;
							bsa.bsa_SrcX = 0;
							bsa.bsa_SrcY = 0;
							bsa.bsa_SrcWidth = PatternTable[Type].Width;
							bsa.bsa_SrcHeight = bmh->bmh_Height;
							bsa.bsa_DestX = 0;
							bsa.bsa_DestY = 0;

							bsa.bsa_XSrcFactor = 1;
							bsa.bsa_XDestFactor = 1;
							bsa.bsa_YSrcFactor = bmh->bmh_Height;
							bsa.bsa_YDestFactor = PatternTable[Type].Height;

							bsa.bsa_SrcBitMap = bm;
							bsa.bsa_DestBitMap = PatternTable[Type].BitMap;

							bsa.bsa_Flags = 0;

							D(bug("custom: scale 0x%lx\n",cbi->BitMap));
							BitMapScale(&bsa);
						}
						else
						{
							D(bug("IPrefs_LoadImage: normal blit 0x%lx\n",PatternTable[Type].BitMap));
							BltBitMap(bm,
							          0,
							          0,
							          PatternTable[Type].BitMap,
							          0,
							          0,
							          bmh->bmh_Width,
							          bmh->bmh_Height,
							          0xC0,
							          0xFF,
							          0);
						}
						DisposeDTObject(MyObject);
						PatternTable[Type].DTObject = NULL;
					}
					else
					{
						D(bug("IPrefs_LoadImage: can't alloc bitmap\n"));
					}
#endif

					if (PatternTable[Type].BitMap)
					{
						SetImage(Type);
					}

				}
				else
				{
					D(bug("IPrefs_LoadImage: can't get bitmap\n"));
				}
			}
			else
			{
				D(bug("IPrefs_LoadImage: can't layout\n"));
			}
		}
		else
		{
			D(bug("IPrefs_LoadImage: can't load <%s>\n",
			      PatternTable[Type].Name));
		}
		ReleaseDTBase();
	}
	else
	{
		D(bug("IPrefs_LoadImage: can't open datatypes.library\n"));
	}

	return MyObject;
}

struct Screen	*ObtainWBScreen(void)
{
	struct Screen	*MyScreen;

	MyScreen	=	LockPubScreen(NULL);

	return(MyScreen);
}

void	ReleaseWBScreen(struct Screen	*MyScreen)
{

	UnlockPubScreen(NULL,MyScreen);
}

void	LoadPatterns(void)
{
	struct Screen	*MyScreen;
	ULONG		Type;

	D(bug("IPrefs_LoadPatterns:\n"));

	if ((MyScreen=ObtainWBScreen()))
	{
		D(bug("IPrefs_LoadPatterns: Screen 0x%lx\n",
		      MyScreen));
		for (Type = WBP_ROOT; Type <= WBP_SCREEN; Type++)
		{
			D(bug("IPrefs_LoadPatterns: Type %ld\n",
			      Type));
			if ((PatternTable[Type].Name) &&
			        (PatternTable[Type].Name[0]))
			{
				if (PatternTable[Type].Action & ACTIONF_UPDATE)
				{
					PatternTable[Type].Action	&=	~ACTIONF_UPDATE;

					D(bug("IPrefs_LoadPatterns: <%s>\n",
					      PatternTable[Type].Name));
					LoadImage(MyScreen,
					          Type);
				}
				else
				{
					D(bug("IPrefs_LoadPatterns: ACTIONF_UPDATE not set..skip\n"));
				}
			}
			else
			{
				D(bug("IPrefs_LoadPatterns: no name\n"));
			}
		}
		ReleaseWBScreen(MyScreen);
	}
	else
	{
		D(bug("IPrefs_LoadPatterns: can't get WBScreen\n"));
	}
}

static void	FreePatternInfo(ULONG	Type)
{
	if (PatternTable[Type].DTObject)
	{
		if (ObtainDTBase())
		{
			DisposeDTObject(PatternTable[Type].DTObject);
			ReleaseDTBase();
			PatternTable[Type].DTObject	=	NULL;
		}
	}
	else
	{
		if (PatternTable[Type].BitMap)
		{
			FreeBitMap(PatternTable[Type].BitMap);
			PatternTable[Type].BitMap	=	NULL;
		}
	}


	if (PatternTable[Type].Name)
	{
		/*
		 * Free Old FileName
		 */
		FreeVec(PatternTable[Type].Name);
		PatternTable[Type].Name	=	NULL;
	}
}

